@@ -55,6 +55,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD reason, LPVOID reserved)
     case DLL_PROCESS_DETACH:
         if (reserved) break;
         release_shared_factory(shared_factory);
+        release_system_fallback_data();
         if (unixlib_handle) UNIX_CALL(process_detach, NULL);
     }
     return TRUE;
@@ -598,7 +599,7 @@ struct dwritefactory
     IDWriteFactory7 IDWriteFactory7_iface;
     LONG refcount;
 
-    IDWriteFontCollection1 *system_collection;
+    IDWriteFontCollection *system_collections[DWRITE_FONT_FAMILY_MODEL_WEIGHT_STRETCH_STYLE + 1];
     IDWriteFontCollection1 *eudc_collection;
     IDWriteGdiInterop1 *gdiinterop;
     IDWriteFontFallback1 *fallback;
@@ -640,6 +641,7 @@ static void release_dwritefactory(struct dwritefactory *factory)
 {
     struct fileloader *fileloader, *fileloader2;
     struct collectionloader *loader, *loader2;
+    unsigned int i;
 
     EnterCriticalSection(&factory->cs);
     release_fontface_cache(&factory->localfontfaces);
@@ -654,8 +656,11 @@ static void release_dwritefactory(struct dwritefactory *factory)
     LIST_FOR_EACH_ENTRY_SAFE(fileloader, fileloader2, &factory->file_loaders, struct fileloader, entry)
         release_fileloader(fileloader);
 
-    if (factory->system_collection)
-        IDWriteFontCollection1_Release(factory->system_collection);
+    for (i = 0; i < ARRAY_SIZE(factory->system_collections); ++i)
+    {
+        if (factory->system_collections[i])
+            IDWriteFontCollection_Release(factory->system_collections[i]);
+    }
     if (factory->eudc_collection)
         IDWriteFontCollection1_Release(factory->eudc_collection);
     if (factory->fallback)
@@ -703,26 +708,35 @@ static struct collectionloader *factory_get_collection_loader(struct dwritefacto
     return found;
 }
 
-static IDWriteFontCollection1 *factory_get_system_collection(struct dwritefactory *factory)
+static HRESULT factory_get_system_collection(struct dwritefactory *factory,
+        DWRITE_FONT_FAMILY_MODEL family_model, REFIID riid, void **out)
 {
-    IDWriteFontCollection1 *collection;
+    IDWriteFontCollection *collection;
     HRESULT hr;
 
-    if (factory->system_collection) {
-        IDWriteFontCollection1_AddRef(factory->system_collection);
-        return factory->system_collection;
+    *out = NULL;
+
+    if (family_model != DWRITE_FONT_FAMILY_MODEL_TYPOGRAPHIC &&
+            family_model != DWRITE_FONT_FAMILY_MODEL_WEIGHT_STRETCH_STYLE)
+    {
+        return E_INVALIDARG;
     }
 
-    if (FAILED(hr = get_system_fontcollection(&factory->IDWriteFactory7_iface, &collection)))
+    if (factory->system_collections[family_model])
+        return IDWriteFontCollection_QueryInterface(factory->system_collections[family_model], riid, out);
+
+    if (FAILED(hr = get_system_fontcollection(&factory->IDWriteFactory7_iface, family_model, &collection)))
     {
         WARN("Failed to create system font collection, hr %#lx.\n", hr);
-        return NULL;
+        return hr;
     }
 
-    if (InterlockedCompareExchangePointer((void **)&factory->system_collection, collection, NULL))
-        IDWriteFontCollection1_Release(collection);
+    if (InterlockedCompareExchangePointer((void **)&factory->system_collections[family_model], collection, NULL))
+        IDWriteFontCollection_Release(collection);
 
-    return factory->system_collection;
+    hr = IDWriteFontCollection_QueryInterface(factory->system_collections[family_model], riid, out);
+    IDWriteFontCollection_Release(factory->system_collections[family_model]);
+    return hr;
 }
 
 static HRESULT WINAPI dwritefactory_QueryInterface(IDWriteFactory7 *iface, REFIID riid, void **obj)
@@ -773,10 +787,17 @@ static ULONG WINAPI dwritefactory_Release(IDWriteFactory7 *iface)
 }
 
 static HRESULT WINAPI dwritefactory_GetSystemFontCollection(IDWriteFactory7 *iface,
-    IDWriteFontCollection **collection, BOOL check_for_updates)
+        IDWriteFontCollection **collection, BOOL check_for_updates)
 {
-    return IDWriteFactory5_GetSystemFontCollection((IDWriteFactory5 *)iface, FALSE, (IDWriteFontCollection1 **)collection,
-            check_for_updates);
+    struct dwritefactory *factory = impl_from_IDWriteFactory7(iface);
+
+    TRACE("%p, %p, %d.\n", iface, collection, check_for_updates);
+
+    if (check_for_updates)
+        FIXME("checking for system font updates not implemented\n");
+
+    return factory_get_system_collection(factory, DWRITE_FONT_FAMILY_MODEL_WEIGHT_STRETCH_STYLE,
+            &IID_IDWriteFontCollection, (void **)collection);
 }
 
 static HRESULT WINAPI dwritefactory_CreateCustomFontCollection(IDWriteFactory7 *iface,
@@ -1167,17 +1188,20 @@ static HRESULT WINAPI dwritefactory_CreateTextFormat(IDWriteFactory7 *iface, WCH
     TRACE("%p, %s, %p, %d, %d, %d, %.8e, %s, %p.\n", iface, debugstr_w(family_name), collection, weight, style, stretch,
         size, debugstr_w(locale), format);
 
+    *format = NULL;
+
     if (collection)
+    {
         IDWriteFontCollection_AddRef(collection);
-    else {
-        collection = (IDWriteFontCollection *)factory_get_system_collection(factory);
-        if (!collection) {
-            *format = NULL;
-            return E_FAIL;
-        }
+    }
+    else if (FAILED(hr = factory_get_system_collection(factory, DWRITE_FONT_FAMILY_MODEL_WEIGHT_STRETCH_STYLE,
+                &IID_IDWriteFontCollection, (void **)&collection)))
+    {
+        return hr;
     }
 
-    hr = create_textformat(family_name, collection, weight, style, stretch, size, locale, format);
+    hr = create_text_format(family_name, collection, weight, style, stretch, size, locale,
+            &IID_IDWriteTextFormat, (void **)format);
     IDWriteFontCollection_Release(collection);
     return hr;
 }
@@ -1611,7 +1635,7 @@ static int __cdecl create_system_fontset_compare(const void *left, const void *r
     return wcsicmp(_l, _r);
 };
 
-static HRESULT create_system_fontset(IDWriteFactory7 *factory, REFIID riid, void **obj)
+HRESULT create_system_fontset(IDWriteFactory7 *factory, REFIID riid, void **obj)
 {
     IDWriteFontSetBuilder2 *builder;
     IDWriteFontSet *fontset;
@@ -1670,11 +1694,12 @@ static HRESULT WINAPI dwritefactory3_CreateFontSetBuilder(IDWriteFactory7 *iface
 }
 
 static HRESULT WINAPI dwritefactory3_CreateFontCollectionFromFontSet(IDWriteFactory7 *iface, IDWriteFontSet *fontset,
-    IDWriteFontCollection1 **collection)
+        IDWriteFontCollection1 **collection)
 {
-    FIXME("%p, %p, %p: stub\n", iface, fontset, collection);
+    TRACE("%p, %p, %p.\n", iface, fontset, collection);
 
-    return E_NOTIMPL;
+    return create_font_collection_from_set(iface, fontset, DWRITE_FONT_FAMILY_MODEL_WEIGHT_STRETCH_STYLE,
+            &IID_IDWriteFontCollection1, (void **)collection);
 }
 
 static HRESULT WINAPI dwritefactory3_GetSystemFontCollection(IDWriteFactory7 *iface, BOOL include_downloadable,
@@ -1690,9 +1715,8 @@ static HRESULT WINAPI dwritefactory3_GetSystemFontCollection(IDWriteFactory7 *if
     if (check_for_updates)
         FIXME("checking for system font updates not implemented\n");
 
-    *collection = factory_get_system_collection(factory);
-
-    return *collection ? S_OK : E_FAIL;
+    return factory_get_system_collection(factory, DWRITE_FONT_FAMILY_MODEL_WEIGHT_STRETCH_STYLE,
+            &IID_IDWriteFontCollection1, (void **)collection);
 }
 
 static HRESULT WINAPI dwritefactory3_GetFontDownloadQueue(IDWriteFactory7 *iface, IDWriteFontDownloadQueue **queue)
@@ -1848,17 +1872,22 @@ static HRESULT WINAPI dwritefactory6_GetSystemFontSet(IDWriteFactory7 *iface, BO
 static HRESULT WINAPI dwritefactory6_GetSystemFontCollection(IDWriteFactory7 *iface, BOOL include_downloadable,
         DWRITE_FONT_FAMILY_MODEL family_model, IDWriteFontCollection2 **collection)
 {
-    FIXME("%p, %d, %d, %p.\n", iface, include_downloadable, family_model, collection);
+    struct dwritefactory *factory = impl_from_IDWriteFactory7(iface);
 
-    return E_NOTIMPL;
+    TRACE("%p, %d, %d, %p.\n", iface, include_downloadable, family_model, collection);
+
+    if (include_downloadable)
+        FIXME("remote fonts are not supported\n");
+
+    return factory_get_system_collection(factory, family_model, &IID_IDWriteFontCollection2, (void **)collection);
 }
 
 static HRESULT WINAPI dwritefactory6_CreateFontCollectionFromFontSet(IDWriteFactory7 *iface, IDWriteFontSet *fontset,
         DWRITE_FONT_FAMILY_MODEL family_model, IDWriteFontCollection2 **collection)
 {
-    FIXME("%p, %p, %d, %p.\n", iface, fontset, family_model, collection);
+    TRACE("%p, %p, %d, %p.\n", iface, fontset, family_model, collection);
 
-    return E_NOTIMPL;
+    return create_font_collection_from_set(iface, fontset, family_model, &IID_IDWriteFontCollection2, (void **)collection);
 }
 
 static HRESULT WINAPI dwritefactory6_CreateFontSetBuilder(IDWriteFactory7 *iface, IDWriteFontSetBuilder2 **builder)
@@ -1868,14 +1897,35 @@ static HRESULT WINAPI dwritefactory6_CreateFontSetBuilder(IDWriteFactory7 *iface
     return create_fontset_builder(iface, builder);
 }
 
-static HRESULT WINAPI dwritefactory6_CreateTextFormat(IDWriteFactory7 *iface, const WCHAR *familyname,
+static HRESULT WINAPI dwritefactory6_CreateTextFormat(IDWriteFactory7 *iface, const WCHAR *family_name,
         IDWriteFontCollection *collection, DWRITE_FONT_AXIS_VALUE const *axis_values, UINT32 num_axis,
-        FLOAT fontsize, const WCHAR *localename, IDWriteTextFormat3 **format)
+        float size, const WCHAR *locale, IDWriteTextFormat3 **format)
 {
-    FIXME("%p, %s, %p, %p, %u, %.8e, %s, %p.\n", iface, debugstr_w(familyname), collection, axis_values, num_axis,
-            fontsize, debugstr_w(localename), format);
+    struct dwritefactory *factory = impl_from_IDWriteFactory7(iface);
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("%p, %s, %p, %p, %u, %.8e, %s, %p.\n", iface, debugstr_w(family_name), collection, axis_values, num_axis,
+            size, debugstr_w(locale), format);
+
+    *format = NULL;
+
+    if (axis_values)
+        FIXME("Axis values are ignored.\n");
+
+    if (collection)
+    {
+        IDWriteFontCollection_AddRef(collection);
+    }
+    else if (FAILED(hr = factory_get_system_collection(factory, DWRITE_FONT_FAMILY_MODEL_TYPOGRAPHIC,
+            &IID_IDWriteFontCollection, (void **)&collection)))
+    {
+        return hr;
+    }
+
+    hr = create_text_format(family_name, collection, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+            DWRITE_FONT_STRETCH_NORMAL, size, locale, &IID_IDWriteTextFormat3, (void **)format);
+    IDWriteFontCollection_Release(collection);
+    return hr;
 }
 
 static HRESULT WINAPI dwritefactory7_GetSystemFontSet(IDWriteFactory7 *iface, BOOL include_downloadable,
@@ -1892,9 +1942,14 @@ static HRESULT WINAPI dwritefactory7_GetSystemFontSet(IDWriteFactory7 *iface, BO
 static HRESULT WINAPI dwritefactory7_GetSystemFontCollection(IDWriteFactory7 *iface, BOOL include_downloadable,
         DWRITE_FONT_FAMILY_MODEL family_model, IDWriteFontCollection3 **collection)
 {
-    FIXME("%p, %d, %d, %p.\n", iface, include_downloadable, family_model, collection);
+    struct dwritefactory *factory = impl_from_IDWriteFactory7(iface);
 
-    return E_NOTIMPL;
+    TRACE("%p, %d, %d, %p.\n", iface, include_downloadable, family_model, collection);
+
+    if (include_downloadable)
+        FIXME("remote fonts are not supported\n");
+
+    return factory_get_system_collection(factory, family_model, &IID_IDWriteFontCollection3, (void **)collection);
 }
 
 static const IDWriteFactory7Vtbl dwritefactoryvtbl =
@@ -2039,10 +2094,6 @@ static void init_dwritefactory(struct dwritefactory *factory, DWRITE_FACTORY_TYP
             &shareddwritefactoryvtbl : &dwritefactoryvtbl;
     factory->refcount = 1;
     factory->localfontfileloader = get_local_fontfile_loader();
-    factory->system_collection = NULL;
-    factory->eudc_collection = NULL;
-    factory->gdiinterop = NULL;
-    factory->fallback = NULL;
 
     list_init(&factory->collection_loaders);
     list_init(&factory->file_loaders);
@@ -2055,7 +2106,10 @@ static void init_dwritefactory(struct dwritefactory *factory, DWRITE_FACTORY_TYP
 void factory_detach_fontcollection(IDWriteFactory7 *iface, IDWriteFontCollection3 *collection)
 {
     struct dwritefactory *factory = impl_from_IDWriteFactory7(iface);
-    InterlockedCompareExchangePointer((void **)&factory->system_collection, NULL, collection);
+    unsigned int i;
+
+    for (i = 0; i < ARRAY_SIZE(factory->system_collections); ++i)
+        InterlockedCompareExchangePointer((void **)&factory->system_collections[i], NULL, collection);
     InterlockedCompareExchangePointer((void **)&factory->eudc_collection, NULL, collection);
     IDWriteFactory7_Release(iface);
 }
