@@ -102,6 +102,7 @@ void wg_parser_stream_seek(struct wg_parser_stream *stream, double rate,
 struct wg_transform *wg_transform_create(const struct wg_format *input_format,
         const struct wg_format *output_format);
 void wg_transform_destroy(struct wg_transform *transform);
+bool wg_transform_set_output_format(struct wg_transform *transform, struct wg_format *format);
 
 unsigned int wg_format_get_max_size(const struct wg_format *format);
 
@@ -140,6 +141,7 @@ HRESULT wg_transform_read_quartz(struct wg_transform *transform, struct wg_sampl
 HRESULT winegstreamer_stream_handler_create(REFIID riid, void **obj);
 
 HRESULT h264_decoder_create(REFIID riid, void **ret);
+HRESULT video_processor_create(REFIID riid, void **ret);
 
 struct wm_stream
 {
@@ -180,8 +182,6 @@ struct wm_reader
     struct wm_stream *streams;
     WORD stream_count;
 
-    IWMReaderCallbackAdvanced *callback_advanced;
-
     const struct wm_reader_ops *ops;
 };
 
@@ -201,7 +201,7 @@ HRESULT wm_reader_get_output_props(struct wm_reader *reader, DWORD output,
         IWMOutputMediaProps **props);
 struct wm_stream *wm_reader_get_stream_by_stream_number(struct wm_reader *reader,
         WORD stream_number);
-HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, WORD stream_number,
+HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, IWMReaderCallbackAdvanced *callback_advanced, WORD stream_number,
         INSSBuffer **ret_sample, QWORD *pts, QWORD *duration, DWORD *flags, WORD *ret_stream_number);
 HRESULT wm_reader_get_stream_selection(struct wm_reader *reader,
         WORD stream_number, WMT_STREAM_SELECTION *selection);
