@@ -263,6 +263,12 @@ struct wg_transform_read_data_params
     HRESULT result;
 };
 
+struct wg_transform_set_output_format_params
+{
+    struct wg_transform *transform;
+    const struct wg_format *format;
+};
+
 enum unix_funcs
 {
     unix_wg_parser_create,
@@ -291,6 +297,7 @@ enum unix_funcs
 
     unix_wg_transform_create,
     unix_wg_transform_destroy,
+    unix_wg_transform_set_output_format,
 
     unix_wg_transform_push_data,
     unix_wg_transform_read_data,
