@@ -116,8 +116,18 @@ static HRESULT WINAPI output_props_GetMediaType(IWMOutputMediaProps *iface, WM_M
 
 static HRESULT WINAPI output_props_SetMediaType(IWMOutputMediaProps *iface, WM_MEDIA_TYPE *mt)
 {
-    FIXME("iface %p, mt %p, stub!\n", iface, mt);
-    return E_NOTIMPL;
+    const struct output_props *props = impl_from_IWMOutputMediaProps(iface);
+
+    TRACE("iface %p, mt %p.\n", iface, mt);
+
+    if (!mt)
+        return E_POINTER;
+
+    if (!IsEqualGUID(&props->mt.majortype, &mt->majortype))
+        return E_FAIL;
+
+    FreeMediaType((AM_MEDIA_TYPE *)&props->mt);
+    return CopyMediaType((AM_MEDIA_TYPE *)&props->mt, (AM_MEDIA_TYPE *)mt);
 }
 
 static HRESULT WINAPI output_props_GetStreamGroupName(IWMOutputMediaProps *iface, WCHAR *name, WORD *len)
@@ -1607,10 +1617,6 @@ HRESULT wm_reader_close(struct wm_reader *reader)
     CloseHandle(reader->read_thread);
     reader->read_thread = NULL;
 
-    if (reader->callback_advanced)
-        IWMReaderCallbackAdvanced_Release(reader->callback_advanced);
-    reader->callback_advanced = NULL;
-
     wg_parser_destroy(reader->wg_parser);
     reader->wg_parser = NULL;
 
@@ -1758,6 +1764,8 @@ HRESULT wm_reader_set_output_props(struct wm_reader *reader, DWORD output,
     struct output_props *props = unsafe_impl_from_IWMOutputMediaProps(props_iface);
     struct wg_format format, pref_format;
     struct wm_stream *stream;
+    HRESULT hr = S_OK;
+    int i;
 
     strmbase_dump_media_type(&props->mt);
 
@@ -1779,9 +1787,39 @@ HRESULT wm_reader_set_output_props(struct wm_reader *reader, DWORD output,
     if (pref_format.major_type != format.major_type)
     {
         /* R.U.S.E sets the type of the wrong stream, apparently by accident. */
+        hr = NS_E_INCOMPATIBLE_FORMAT;
+    }
+    else switch (pref_format.major_type)
+    {
+        case WG_MAJOR_TYPE_AUDIO:
+            if (format.u.audio.format == WG_AUDIO_FORMAT_UNKNOWN)
+                hr = NS_E_AUDIO_CODEC_NOT_INSTALLED;
+            else if (format.u.audio.channels > pref_format.u.audio.channels)
+                hr = NS_E_AUDIO_CODEC_NOT_INSTALLED;
+            break;
+
+        case WG_MAJOR_TYPE_VIDEO:
+            for (i = 0; i < ARRAY_SIZE(video_formats); ++i)
+                if (format.u.video.format == video_formats[i])
+                    break;
+            if (i == ARRAY_SIZE(video_formats))
+                hr = NS_E_INVALID_OUTPUT_FORMAT;
+            else if (pref_format.u.video.width != format.u.video.width)
+                hr = NS_E_INVALID_OUTPUT_FORMAT;
+            else if (pref_format.u.video.height != format.u.video.height)
+                hr = NS_E_INVALID_OUTPUT_FORMAT;
+            break;
+
+        default:
+            hr = NS_E_INCOMPATIBLE_FORMAT;
+            break;
+    }
+
+    if (FAILED(hr))
+    {
+        WARN("Unsupported media type, returning %#lx.\n", hr);
         LeaveCriticalSection(&reader->cs);
-        WARN("Major types don't match; returning NS_E_INCOMPATIBLE_FORMAT.\n");
-        return NS_E_INCOMPATIBLE_FORMAT;
+        return hr;
     }
 
     stream->format = format;
@@ -1871,10 +1909,9 @@ static WORD get_earliest_buffer(struct wm_reader *reader, struct wg_parser_buffe
     return stream_number;
 }
 
-HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, WORD stream_number,
+HRESULT wm_reader_get_stream_sample(struct wm_reader *reader, IWMReaderCallbackAdvanced *callback_advanced, WORD stream_number,
         INSSBuffer **ret_sample, QWORD *pts, QWORD *duration, DWORD *flags, WORD *ret_stream_number)
 {
-    IWMReaderCallbackAdvanced *callback_advanced = reader->callback_advanced;
     struct wg_parser_stream *wg_stream;
     struct wg_parser_buffer wg_buffer;
     struct wm_stream *stream;
