@@ -204,6 +204,7 @@
     SYSCALL_ENTRY( NtUserIsClipboardFormatAvailable ) \
     SYSCALL_ENTRY( NtUserKillTimer ) \
     SYSCALL_ENTRY( NtUserLockWindowUpdate ) \
+    SYSCALL_ENTRY( NtUserLogicalToPerMonitorDPIPhysicalPoint ) \
     SYSCALL_ENTRY( NtUserMapVirtualKeyEx ) \
     SYSCALL_ENTRY( NtUserMenuItemFromPoint ) \
     SYSCALL_ENTRY( NtUserMessageCall ) \
@@ -215,6 +216,7 @@
     SYSCALL_ENTRY( NtUserOpenInputDesktop ) \
     SYSCALL_ENTRY( NtUserOpenWindowStation ) \
     SYSCALL_ENTRY( NtUserPeekMessage ) \
+    SYSCALL_ENTRY( NtUserPerMonitorDPIPhysicalToLogicalPoint ) \
     SYSCALL_ENTRY( NtUserPostMessage ) \
     SYSCALL_ENTRY( NtUserPostThreadMessage ) \
     SYSCALL_ENTRY( NtUserQueryInputContext ) \
