@@ -23,17 +23,11 @@
 
 #include <stdarg.h>
 #include <stdlib.h>
+#include <io.h>
 #include <windef.h>
 #include <winbase.h>
 #include <wine/debug.h>
 
-#ifdef __WINE_CONFIG_H
-#error config.h should not be used in Wine tests
-#endif
-#ifdef __WINE_WINE_UNICODE_H
-#error wine/unicode.h should not be used in Wine tests
-#endif
-
 #ifndef INVALID_FILE_ATTRIBUTES
 #define INVALID_FILE_ATTRIBUTES  (~0u)
 #endif
@@ -50,6 +44,9 @@ extern int winetest_time;
 /* running in interactive mode? */
 extern int winetest_interactive;
 
+/* always count flaky tests as successful (BOOL) */
+extern int winetest_allow_flaky;
+
 /* report successful tests (BOOL) */
 extern int winetest_report_success;
 
@@ -65,6 +62,9 @@ extern void winetest_ignore_exceptions( BOOL ignore );
 extern void winetest_start_todo( int is_todo );
 extern int winetest_loop_todo(void);
 extern void winetest_end_todo(void);
+extern void winetest_start_flaky( int is_flaky );
+extern int winetest_loop_flaky(void);
+extern void winetest_end_flaky(void);
 extern int winetest_get_mainargs( char*** pargv );
 extern LONG winetest_get_failures(void);
 extern void winetest_add_failures( LONG new_failures );
@@ -117,6 +117,13 @@ extern void winetest_pop_context(void);
 #define trace    trace_(__FILE__, __LINE__)
 #define wait_child_process wait_child_process_(__FILE__, __LINE__)
 
+#define flaky_if(is_flaky) for (winetest_start_flaky(is_flaky); \
+                                winetest_loop_flaky(); \
+                                winetest_end_flaky())
+#define flaky                   flaky_if(TRUE)
+#define flaky_wine              flaky_if(!strcmp(winetest_platform, "wine"))
+#define flaky_wine_if(is_flaky) flaky_if((is_flaky) && !strcmp(winetest_platform, "wine"))
+
 #define todo_if(is_todo) for (winetest_start_todo(is_todo); \
                               winetest_loop_todo(); \
                               winetest_end_todo())
@@ -200,12 +207,18 @@ int winetest_interactive = 0;
 /* current platform */
 const char *winetest_platform = "windows";
 
+/* always count flaky tests as successful (BOOL) */
+int winetest_allow_flaky;
+
 /* report successful tests (BOOL) */
 int winetest_report_success = 0;
 
 /* silence todos and skips above this threshold */
 int winetest_mute_threshold = 42;
 
+/* use ANSI escape codes for output coloring */
+static int winetest_color;
+
 /* passing arguments around */
 static int winetest_argc;
 static char** winetest_argv;
@@ -214,6 +227,7 @@ static const struct test *current_test; /* test currently being run */
 
 static LONG successes;       /* number of successful tests */
 static LONG failures;        /* number of failures */
+static LONG flaky_failures;  /* number of failures inside flaky block */
 static LONG skipped;         /* number of skipped test chunks */
 static LONG todo_successes;  /* number of successful tests inside todo block */
 static LONG todo_failures;   /* number of failures inside todo block */
@@ -229,6 +243,8 @@ struct tls_data
 {
     const char* current_file;        /* file of current check */
     int current_line;                /* line of current check */
+    unsigned int flaky_level;        /* current flaky nesting level */
+    int flaky_do_loop;
     unsigned int todo_level;         /* current todo nesting level */
     int todo_do_loop;
     char *str_pos;                   /* position in debug buffer */
@@ -284,6 +300,15 @@ const char *winetest_elapsed(void)
     return wine_dbg_sprintf( "%.3f", (now - winetest_start_time) / 1000.0);
 }
 
+static const char color_reset[] = "\x1b[0m";
+static const char color_dark_red[] = "\x1b[31m";
+static const char color_dark_purple[] = "\x1b[35m";
+static const char color_green[] = "\x1b[32m";
+static const char color_yellow[] = "\x1b[33m";
+static const char color_blue[] = "\x1b[34m";
+static const char color_bright_red[] = "\x1b[1;91m";
+static const char color_bright_purple[] = "\x1b[1;95m";
+
 static void winetest_printf( const char *msg, ... ) __WINE_PRINTF_ATTR(1,2);
 static void winetest_printf( const char *msg, ... )
 {
@@ -355,9 +380,21 @@ int winetest_vok( int condition, const char *msg, va_list args )
     {
         if (condition)
         {
-            winetest_print_context( "Test succeeded inside todo block: " );
-            vprintf(msg, args);
-            InterlockedIncrement(&todo_failures);
+            if (data->flaky_level)
+            {
+                if (winetest_color) printf( color_dark_purple );
+                winetest_print_context( "Test succeeded inside flaky todo block: " );
+                vprintf(msg, args);
+                InterlockedIncrement(&flaky_failures);
+            }
+            else
+            {
+                if (winetest_color) printf( color_dark_red );
+                winetest_print_context( "Test succeeded inside todo block: " );
+                vprintf(msg, args);
+                InterlockedIncrement(&todo_failures);
+            }
+            if (winetest_color) printf( color_reset );
             return 0;
         }
         else
@@ -367,8 +404,10 @@ int winetest_vok( int condition, const char *msg, va_list args )
             {
                 if (winetest_debug > 0)
                 {
+                    if (winetest_color) printf( color_yellow );
                     winetest_print_context( "Test marked todo: " );
                     vprintf(msg, args);
+                    if (winetest_color) printf( color_reset );
                 }
                 InterlockedIncrement(&todo_successes);
             }
@@ -381,9 +420,21 @@ int winetest_vok( int condition, const char *msg, va_list args )
     {
         if (!condition)
         {
-            winetest_print_context( "Test failed: " );
-            vprintf(msg, args);
-            InterlockedIncrement(&failures);
+            if (data->flaky_level)
+            {
+                if (winetest_color) printf( color_bright_purple );
+                winetest_print_context( "Test marked flaky: " );
+                vprintf(msg, args);
+                InterlockedIncrement(&flaky_failures);
+            }
+            else
+            {
+                if (winetest_color) printf( color_bright_red );
+                winetest_print_context( "Test failed: " );
+                vprintf(msg, args);
+                InterlockedIncrement(&failures);
+            }
+            if (winetest_color) printf( color_reset );
             return 0;
         }
         else
@@ -391,7 +442,9 @@ int winetest_vok( int condition, const char *msg, va_list args )
             if (winetest_report_success ||
                 (winetest_time && GetTickCount() >= winetest_last_time + 1000))
             {
+                if (winetest_color) printf( color_green );
                 winetest_printf("Test succeeded\n");
+                if (winetest_color) printf( color_reset );
             }
             InterlockedIncrement(&successes);
             return 1;
@@ -429,8 +482,10 @@ void winetest_vskip( const char *msg, va_list args )
 {
     if (winetest_add_line() < winetest_mute_threshold)
     {
+        if (winetest_color) printf( color_blue );
         winetest_print_context( "Tests skipped: " );
         vprintf(msg, args);
+        if (winetest_color) printf( color_reset );
         InterlockedIncrement(&skipped);
     }
     else
@@ -456,6 +511,27 @@ void winetest_win_skip( const char *msg, ... )
     va_end(valist);
 }
 
+void winetest_start_flaky( int is_flaky )
+{
+    struct tls_data *data = get_tls_data();
+    data->flaky_level = (data->flaky_level << 1) | (is_flaky != 0);
+    data->flaky_do_loop = 1;
+}
+
+int winetest_loop_flaky(void)
+{
+    struct tls_data *data = get_tls_data();
+    int do_flaky = data->flaky_do_loop;
+    data->flaky_do_loop = 0;
+    return do_flaky;
+}
+
+void winetest_end_flaky(void)
+{
+    struct tls_data *data = get_tls_data();
+    data->flaky_level >>= 1;
+}
+
 void winetest_start_todo( int is_todo )
 {
     struct tls_data *data = get_tls_data();
@@ -601,14 +677,16 @@ static int run_test( const char *name )
             printf( "%04x:%s:%s Silenced %d todos, %d skips and %d traces.\n",
                     (UINT)GetCurrentProcessId(), test->name, winetest_elapsed(),
                     (UINT)muted_todo_successes, (UINT)muted_skipped, (UINT)muted_traces);
-        printf( "%04x:%s:%s %d tests executed (%d marked as todo, %d %s), %d skipped.\n",
+        printf( "%04x:%s:%s %d tests executed (%d marked as todo, %d as flaky, %d %s), %d skipped.\n",
                 (UINT)GetCurrentProcessId(), test->name, winetest_elapsed(),
-                (UINT)(successes + failures + todo_successes + todo_failures),
-                (UINT)todo_successes, (UINT)(failures + todo_failures),
+                (UINT)(successes + failures + flaky_failures + todo_successes + todo_failures),
+                (UINT)todo_successes, (UINT)flaky_failures, (UINT)(failures + todo_failures),
                 (failures + todo_failures != 1) ? "failures" : "failure",
                 (UINT)skipped );
     }
-    status = (failures + todo_failures < 255) ? failures + todo_failures : 255;
+    status = failures + todo_failures;
+    if (!winetest_allow_flaky) status += flaky_failures;
+    if (status > 255) status = 255;
     return status;
 }
 
@@ -663,8 +741,24 @@ int main( int argc, char **argv )
     else if (running_under_wine())
         winetest_platform = "wine";
 
+    if (GetEnvironmentVariableA( "WINETEST_COLOR", p, sizeof(p) ))
+    {
+        BOOL automode = !strcmp(p, "auto");
+        winetest_color = automode ? isatty( fileno( stdout ) ) : atoi(p);
+        /* enable ANSI support for Windows console */
+        if (winetest_color)
+        {
+            HANDLE hOutput = (HANDLE)_get_osfhandle( fileno( stdout ) );
+            DWORD mode;
+            if (GetConsoleMode( hOutput, &mode ) &&
+                !SetConsoleMode( hOutput, mode | ENABLE_PROCESSED_OUTPUT | ENABLE_VIRTUAL_TERMINAL_PROCESSING ) &&
+                automode)
+                winetest_color = 0;
+        }
+    }
     if (GetEnvironmentVariableA( "WINETEST_DEBUG", p, sizeof(p) )) winetest_debug = atoi(p);
     if (GetEnvironmentVariableA( "WINETEST_INTERACTIVE", p, sizeof(p) )) winetest_interactive = atoi(p);
+    if (GetEnvironmentVariableA( "WINETEST_ALLOW_FLAKY", p, sizeof(p) )) winetest_allow_flaky = atoi(p);
     if (GetEnvironmentVariableA( "WINETEST_REPORT_SUCCESS", p, sizeof(p) )) winetest_report_success = atoi(p);
     if (GetEnvironmentVariableA( "WINETEST_TIME", p, sizeof(p) )) winetest_time = atoi(p);
     winetest_last_time = winetest_start_time = GetTickCount();
