@@ -64,10 +64,12 @@ static const char testexe[] = "_test.exe";
 static char build_id[64];
 static BOOL is_wow64;
 static int failures;
+static int quiet_mode;
 
 /* filters for running only specific tests */
-static char *filters[64];
-static unsigned int nb_filters = 0;
+static char **filters;
+static unsigned int nb_filters;
+static unsigned int alloc_filters;
 static BOOL exclude_tests = FALSE;
 
 /* Needed to check for .NET dlls */
@@ -109,6 +111,89 @@ static BOOL test_filtered_out( LPCSTR module, LPCSTR testname )
     return !exclude_tests;
 }
 
+static void add_filter( const char *name )
+{
+    if (name[0] == '@')
+    {
+        char *p, *str, buffer[256];
+        FILE *f = fopen( name + 1, "rt" );
+        if (!f) return;
+
+        while (fgets( buffer, sizeof(buffer), f ))
+        {
+            p = buffer;
+            while (*p == ' ' || *p == '\t') p++;
+            if (*p == '#') continue;
+            str = p;
+            while (*p && *p != ' ' && *p != '\t' && *p != '\r' && *p != '\n') p++;
+            *p = 0;
+            add_filter( str );
+        }
+        fclose( f );
+        return;
+    }
+
+    if (nb_filters >= alloc_filters)
+    {
+        alloc_filters = max( alloc_filters * 2, 64 );
+        filters = xrealloc( filters, alloc_filters * sizeof(*filters) );
+    }
+    filters[nb_filters++] = xstrdup(name);
+}
+
+static HANDLE create_output_file( const char *name )
+{
+    SECURITY_ATTRIBUTES sa;
+    HANDLE file;
+
+    /* make handle inheritable */
+    sa.nLength = sizeof(sa);
+    sa.lpSecurityDescriptor = NULL;
+    sa.bInheritHandle = TRUE;
+
+    file = CreateFileA( name, GENERIC_READ|GENERIC_WRITE,
+                        FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                        &sa, CREATE_ALWAYS, 0, NULL );
+
+    if (file == INVALID_HANDLE_VALUE && GetLastError() == ERROR_INVALID_PARAMETER)
+    {
+        /* FILE_SHARE_DELETE not supported on win9x */
+        file = CreateFileA( name, GENERIC_READ|GENERIC_WRITE,
+                            FILE_SHARE_READ | FILE_SHARE_WRITE,
+                            &sa, CREATE_ALWAYS, 0, NULL );
+    }
+    return file;
+}
+
+static HANDLE create_temp_file( char name[MAX_PATH] )
+{
+    char tmpdir[MAX_PATH];
+
+    if (!GetTempPathA( MAX_PATH, tmpdir ) ||
+        !GetTempFileNameA( tmpdir, "out", 0, name ))
+        report (R_FATAL, "Can't name temp file.");
+
+    return create_output_file( name );
+}
+
+static void close_temp_file( const char *name, HANDLE file )
+{
+    CloseHandle( file );
+    DeleteFileA( name );
+}
+
+static char *flush_temp_file( const char *name, HANDLE file, DWORD *retsize )
+{
+    DWORD size = SetFilePointer( file, 0, NULL, FILE_CURRENT );
+    char *buffer = xalloc( size + 1 );
+
+    SetFilePointer( file, 0, NULL, FILE_BEGIN );
+    if (!ReadFile( file, buffer, size, retsize, NULL )) *retsize = 0;
+    close_temp_file( name, file );
+    buffer[*retsize] = 0;
+    return buffer;
+}
+
 static char * get_file_version(char * file_name)
 {
     static char version[32];
@@ -117,29 +202,26 @@ static char * get_file_version(char * file_name)
 
     size = GetFileVersionInfoSizeA(file_name, &handle);
     if (size) {
-        char * data = heap_alloc(size);
-        if (data) {
-            if (GetFileVersionInfoA(file_name, handle, size, data)) {
-                static const char backslash[] = "\\";
-                VS_FIXEDFILEINFO *pFixedVersionInfo;
-                UINT len;
-                if (VerQueryValueA(data, backslash, (LPVOID *)&pFixedVersionInfo, &len)) {
-                    sprintf(version, "%d.%d.%d.%d",
-                            pFixedVersionInfo->dwFileVersionMS >> 16,
-                            pFixedVersionInfo->dwFileVersionMS & 0xffff,
-                            pFixedVersionInfo->dwFileVersionLS >> 16,
-                            pFixedVersionInfo->dwFileVersionLS & 0xffff);
-                } else
-                    sprintf(version, "version not found");
+        char * data = xalloc(size);
+        if (GetFileVersionInfoA(file_name, handle, size, data)) {
+            static const char backslash[] = "\\";
+            VS_FIXEDFILEINFO *pFixedVersionInfo;
+            UINT len;
+            if (VerQueryValueA(data, backslash, (LPVOID *)&pFixedVersionInfo, &len)) {
+                sprintf(version, "%ld.%ld.%ld.%ld",
+                        pFixedVersionInfo->dwFileVersionMS >> 16,
+                        pFixedVersionInfo->dwFileVersionMS & 0xffff,
+                        pFixedVersionInfo->dwFileVersionLS >> 16,
+                        pFixedVersionInfo->dwFileVersionLS & 0xffff);
             } else
-                sprintf(version, "version error %u", GetLastError());
-            heap_free(data);
+                sprintf(version, "version not found");
         } else
-            sprintf(version, "version error %u", ERROR_OUTOFMEMORY);
+            sprintf(version, "version error %lu", GetLastError());
+        free(data);
     } else if (GetLastError() == ERROR_FILE_NOT_FOUND)
         sprintf(version, "dll is missing");
     else
-        sprintf(version, "version not present %u", GetLastError());
+        sprintf(version, "version not present %lu", GetLastError());
 
     return version;
 }
@@ -237,16 +319,10 @@ static int running_as_admin (void)
     /* Get the group info from the token */
     groups_size = 0;
     GetTokenInformation(token, TokenGroups, NULL, 0, &groups_size);
-    groups = heap_alloc(groups_size);
-    if (groups == NULL)
-    {
-        CloseHandle(token);
-        FreeSid(administrators);
-        return -1;
-    }
+    groups = xalloc(groups_size);
     if (! GetTokenInformation(token, TokenGroups, groups, groups_size, &groups_size))
     {
-        heap_free(groups);
+        free(groups);
         CloseHandle(token);
         FreeSid(administrators);
         return -1;
@@ -258,14 +334,14 @@ static int running_as_admin (void)
     {
         if (EqualSid(groups->Groups[group_index].Sid, administrators))
         {
-            heap_free(groups);
+            free(groups);
             FreeSid(administrators);
             return 1;
         }
     }
 
     /* If we end up here we didn't find the Administrators group */
-    heap_free(groups);
+    free(groups);
     FreeSid(administrators);
     return 0;
 }
@@ -326,9 +402,7 @@ static BOOL is_stub_dll(const char *filename)
     size = GetFileVersionInfoSizeA(filename, &ver);
     if (!size) return FALSE;
 
-    data = HeapAlloc(GetProcessHeap(), 0, size);
-    if (!data) return FALSE;
-
+    data = xalloc(size);
     if (GetFileVersionInfoA(filename, ver, size, data))
     {
         char buf[256];
@@ -337,11 +411,39 @@ static BOOL is_stub_dll(const char *filename)
         if (VerQueryValueA(data, buf, (void**)&p, &size))
             isstub = !lstrcmpiA("wcodstub.dll", p);
     }
-    HeapFree(GetProcessHeap(), 0, data);
+    free(data);
 
     return isstub;
 }
 
+static int disable_crash_dialog(void)
+{
+    HKEY key;
+    DWORD type, data, size;
+    int ret = 0;
+
+    if (RegCreateKeyA( HKEY_CURRENT_USER, "Software\\Wine\\WineDbg", &key )) return 0;
+    size = sizeof(data);
+    if (RegQueryValueExA( key, "ShowCrashDialog", NULL, &type, (BYTE *)&data, &size )) ret = 1;
+    else if (type != REG_DWORD || data) ret = 2;
+    data = 0;
+    RegSetValueExA( key, "ShowCrashDialog", 0, REG_DWORD, (BYTE *)&data, sizeof(data) );
+    RegCloseKey( key );
+    return ret;
+}
+
+static void restore_crash_dialog( int prev )
+{
+    HKEY key;
+    DWORD data = 1;
+
+    if (!prev) return;
+    if (RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\WineDbg", &key )) return;
+    if (prev == 1) RegDeleteKeyValueA( key, NULL, "ShowCrashDialog" );
+    else RegSetValueExA( key, "ShowCrashDialog", 0, REG_DWORD, (BYTE *)&data, sizeof(data) );
+    RegCloseKey( key );
+}
+
 static void print_version (void)
 {
 #ifdef __i386__
@@ -477,6 +579,8 @@ static void print_language(void)
         xprintf ("    UserDefaultUILanguage=%04x\n", pGetUserDefaultUILanguage());
     if (pGetThreadUILanguage)
         xprintf ("    ThreadUILanguage=%04x\n", pGetThreadUILanguage());
+    xprintf ("    Country=%d\n", GetUserGeoID(GEOCLASS_NATION));
+    xprintf ("    ACP=%d\n", GetACP());
 }
 
 static inline BOOL is_dot_dir(const char* x)
@@ -559,12 +663,12 @@ extract_test (struct wine_test *test, const char *dir, LPSTR res_name)
     code = extract_rcdata (res_name, "TESTRES", &size);
     if (!code) report (R_FATAL, "Can't find test resource %s: %d",
                        res_name, GetLastError ());
-    test->name = heap_strdup( res_name );
+    test->name = xstrdup( res_name );
     test->exename = strmake (NULL, "%s\\%s", dir, test->name);
     exepos = strstr (test->name, testexe);
     if (!exepos) report (R_FATAL, "Not an .exe file: %s", test->name);
     *exepos = 0;
-    test->name = heap_realloc (test->name, exepos - test->name + 1);
+    test->name = xrealloc(test->name, exepos - test->name + 1);
     report (R_STEP, "Extracting: %s", test->name);
 
     hfile = CreateFileA(test->exename, GENERIC_READ | GENERIC_WRITE, 0, NULL,
@@ -595,15 +699,9 @@ static DWORD wait_process( HANDLE process, DWORD timeout )
 
 static void append_path( const char *path)
 {
-    char *newpath;
-
-    newpath = heap_alloc(strlen(curpath) + 1 + strlen(path) + 1);
-    strcpy(newpath, curpath);
-    strcat(newpath, ";");
-    strcat(newpath, path);
+    char *newpath = strmake( NULL, "%s;%s", curpath, path );
     SetEnvironmentVariableA("PATH", newpath);
-
-    heap_free(newpath);
+    free(newpath);
 }
 
 /* Run a command for MS milliseconds.  If OUT != NULL, also redirect
@@ -620,15 +718,11 @@ run_ex (char *cmd, HANDLE out_file, const char *tempdir, DWORD ms, BOOL nocritic
     DWORD wait, status, flags;
     UINT old_errmode;
 
-    /* Flush to disk so we know which test caused Windows to crash if it does */
-    if (out_file)
-        FlushFileBuffers(out_file);
-
     GetStartupInfoA (&si);
     si.dwFlags    = STARTF_USESTDHANDLES;
     si.hStdInput  = GetStdHandle( STD_INPUT_HANDLE );
-    si.hStdOutput = out_file ? out_file : GetStdHandle( STD_OUTPUT_HANDLE );
-    si.hStdError  = out_file ? out_file : GetStdHandle( STD_ERROR_HANDLE );
+    si.hStdOutput = out_file;
+    si.hStdError  = out_file;
     if (nocritical)
     {
         old_errmode = SetErrorMode(0);
@@ -692,40 +786,15 @@ get_subtests (const char *tempdir, struct wine_test *test, LPSTR res_name)
     char *cmd;
     HANDLE subfile;
     DWORD err, total;
-    char buffer[8192], *index;
+    char *buffer, *index;
     static const char header[] = "Valid test names:";
     int status, allocated;
-    char tmpdir[MAX_PATH], subname[MAX_PATH];
-    SECURITY_ATTRIBUTES sa;
+    char subname[MAX_PATH];
 
     test->subtest_count = 0;
 
-    if (!GetTempPathA( MAX_PATH, tmpdir ) ||
-        !GetTempFileNameA( tmpdir, "sub", 0, subname ))
-        report (R_FATAL, "Can't name subtests file.");
-
-    /* make handle inheritable */
-    sa.nLength = sizeof(sa);
-    sa.lpSecurityDescriptor = NULL;
-    sa.bInheritHandle = TRUE;
-
-    subfile = CreateFileA( subname, GENERIC_READ|GENERIC_WRITE,
-                           FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
-                           &sa, CREATE_ALWAYS, 0, NULL );
-
-    if ((subfile == INVALID_HANDLE_VALUE) &&
-        (GetLastError() == ERROR_INVALID_PARAMETER)) {
-        /* FILE_SHARE_DELETE not supported on win9x */
-        subfile = CreateFileA( subname, GENERIC_READ|GENERIC_WRITE,
-                           FILE_SHARE_READ | FILE_SHARE_WRITE,
-                           &sa, CREATE_ALWAYS, 0, NULL );
-    }
-    if (subfile == INVALID_HANDLE_VALUE) {
-        err = GetLastError();
-        report (R_ERROR, "Can't open subtests output of %s: %u",
-                test->name, GetLastError());
-        goto quit;
-    }
+    subfile = create_temp_file( subname );
+    if (subfile == INVALID_HANDLE_VALUE) return GetLastError();
 
     cmd = strmake (NULL, "%s --list", test->exename);
     if (test->maindllpath) {
@@ -738,7 +807,7 @@ get_subtests (const char *tempdir, struct wine_test *test, LPSTR res_name)
         /* Restore PATH again */
         SetEnvironmentVariableA("PATH", curpath);
     }
-    heap_free (cmd);
+    free(cmd);
 
     if (status)
     {
@@ -746,50 +815,33 @@ get_subtests (const char *tempdir, struct wine_test *test, LPSTR res_name)
             report (R_ERROR, "Cannot run %s error %u", test->exename, err);
         else
             err = status;
-        CloseHandle( subfile );
-        goto quit;
+        close_temp_file( subname, subfile );
+        return err;
     }
 
-    SetFilePointer( subfile, 0, NULL, FILE_BEGIN );
-    ReadFile( subfile, buffer, sizeof(buffer), &total, NULL );
-    CloseHandle( subfile );
-    if (sizeof buffer == total) {
-        report (R_ERROR, "Subtest list of %s too big.",
-                test->name, sizeof buffer);
-        err = ERROR_OUTOFMEMORY;
-        goto quit;
-    }
-    buffer[total] = 0;
-
+    buffer = flush_temp_file( subname, subfile, &total );
     index = strstr (buffer, header);
     if (!index) {
         report (R_ERROR, "Can't parse subtests output of %s",
                 test->name);
-        err = ERROR_INTERNAL_ERROR;
-        goto quit;
+        return ERROR_INTERNAL_ERROR;
     }
     index += sizeof header;
 
     allocated = 10;
-    test->subtests = heap_alloc (allocated * sizeof(char*));
+    test->subtests = xalloc(allocated * sizeof(char*));
     index = strtok (index, whitespace);
     while (index) {
         if (test->subtest_count == allocated) {
             allocated *= 2;
-            test->subtests = heap_realloc (test->subtests,
-                                           allocated * sizeof(char*));
+            test->subtests = xrealloc(test->subtests, allocated * sizeof(char*));
         }
-        test->subtests[test->subtest_count++] = heap_strdup(index);
+        test->subtests[test->subtest_count++] = xstrdup(index);
         index = strtok (NULL, whitespace);
     }
-    test->subtests = heap_realloc (test->subtests,
-                                   test->subtest_count * sizeof(char*));
-    err = 0;
-
- quit:
-    if (!DeleteFileA (subname))
-        report (R_WARNING, "Can't delete file '%s': %u", subname, GetLastError());
-    return err;
+    test->subtests = xrealloc(test->subtests, test->subtest_count * sizeof(char*));
+    free( buffer );
+    return 0;
 }
 
 static void
@@ -807,13 +859,28 @@ run_test (struct wine_test* test, const char* subtest, HANDLE out_file, const ch
     else
     {
         int status;
-        DWORD pid, start = GetTickCount();
+        DWORD pid, size, start = GetTickCount();
         char *cmd = strmake (NULL, "%s %s", test->exename, subtest);
         report (R_STEP, "Running: %s:%s", test->name, subtest);
         xprintf ("%s:%s start %s\n", test->name, subtest, file);
-        status = run_ex (cmd, out_file, tempdir, 120000, FALSE, &pid);
+        /* Flush to disk so we know which test caused Windows to crash if it does */
+        FlushFileBuffers(out_file);
+        if (quiet_mode > 1)
+        {
+            char *data, tmpname[MAX_PATH];
+            HANDLE tmpfile = create_temp_file( tmpname );
+            status = run_ex (cmd, tmpfile, tempdir, 120000, FALSE, &pid);
+            if (status)
+            {
+                data = flush_temp_file( tmpname, tmpfile, &size );
+                WriteFile( out_file, data, size, &size, NULL );
+                free( data );
+            }
+            else close_temp_file( tmpname, tmpfile );
+        }
+        else status = run_ex (cmd, out_file, tempdir, 120000, FALSE, &pid);
         if (status == -2) status = -GetLastError();
-        heap_free (cmd);
+        free(cmd);
         xprintf ("%s:%s:%04x done (%d) in %ds\n", test->name, subtest, pid, status, (GetTickCount()-start)/1000);
         if (status) failures++;
     }
@@ -865,7 +932,7 @@ static HMODULE load_com_dll(const char *name, char **path, char *filename)
 
     if(!get_main_clsid(name, &clsid)) return NULL;
 
-    sprintf(keyname, "CLSID\\{%08x-%04x-%04x-%02x%2x-%02x%2x%02x%2x%02x%2x}\\InprocServer32",
+    sprintf(keyname, "CLSID\\{%08lx-%04x-%04x-%02x%2x-%02x%2x%02x%2x%02x%2x}\\InprocServer32",
             clsid.Data1, clsid.Data2, clsid.Data3, clsid.Data4[0], clsid.Data4[1],
             clsid.Data4[2], clsid.Data4[3], clsid.Data4[4], clsid.Data4[5],
             clsid.Data4[6], clsid.Data4[7]);
@@ -880,7 +947,7 @@ static HMODULE load_com_dll(const char *name, char **path, char *filename)
                 strcpy( filename, dllname );
                 p = strrchr(dllname, '\\');
                 if (p) *p = 0;
-                *path = heap_strdup( dllname );
+                *path = xstrdup( dllname );
             }
         }
         RegCloseKey(hkey);
@@ -896,7 +963,7 @@ static void get_dll_path(HMODULE dll, char **path, char *filename)
     GetModuleFileNameA(dll, dllpath, MAX_PATH);
     strcpy(filename, dllpath);
     *strrchr(dllpath, '\\') = '\0';
-    *path = heap_strdup( dllpath );
+    *path = xstrdup( dllpath );
 }
 
 static BOOL CALLBACK
@@ -1023,7 +1090,6 @@ run_tests (char *logname, char *outdir)
     int i;
     char *strres, *eol, *nextline;
     DWORD strsize;
-    SECURITY_ATTRIBUTES sa;
     char tmppath[MAX_PATH], tempdir[MAX_PATH+4];
     BOOL newdir;
     DWORD needed;
@@ -1031,7 +1097,7 @@ run_tests (char *logname, char *outdir)
 
     /* Get the current PATH only once */
     needed = GetEnvironmentVariableA("PATH", NULL, 0);
-    curpath = heap_alloc(needed);
+    curpath = xalloc(needed);
     GetEnvironmentVariableA("PATH", curpath, needed);
 
     SetErrorMode (SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX);
@@ -1039,31 +1105,19 @@ run_tests (char *logname, char *outdir)
     if (!GetTempPathA( MAX_PATH, tmppath ))
         report (R_FATAL, "Can't name temporary dir (check %%TEMP%%).");
 
-    if (!logname) {
+    if (logname)
+    {
+        if (!strcmp(logname, "-")) logfile = GetStdHandle( STD_OUTPUT_HANDLE );
+        else logfile = create_output_file( logname );
+    }
+    else
+    {
         static char tmpname[MAX_PATH];
-        if (!GetTempFileNameA( tmppath, "res", 0, tmpname ))
-            report (R_FATAL, "Can't name logfile.");
+        logfile = create_temp_file( tmpname );
         logname = tmpname;
     }
     report (R_OUT, logname);
 
-    /* make handle inheritable */
-    sa.nLength = sizeof(sa);
-    sa.lpSecurityDescriptor = NULL;
-    sa.bInheritHandle = TRUE;
-
-    logfile = strcmp(logname, "-") == 0 ? GetStdHandle( STD_OUTPUT_HANDLE ) :
-              CreateFileA( logname, GENERIC_READ|GENERIC_WRITE,
-                           FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
-                           &sa, CREATE_ALWAYS, 0, NULL );
-
-    if ((logfile == INVALID_HANDLE_VALUE) &&
-        (GetLastError() == ERROR_INVALID_PARAMETER)) {
-        /* FILE_SHARE_DELETE not supported on win9x */
-        logfile = CreateFileA( logname, GENERIC_READ|GENERIC_WRITE,
-                           FILE_SHARE_READ | FILE_SHARE_WRITE,
-                           &sa, CREATE_ALWAYS, 0, NULL );
-    }
     if (logfile == INVALID_HANDLE_VALUE)
         report (R_FATAL, "Could not open logfile: %u", GetLastError());
 
@@ -1118,7 +1172,7 @@ run_tests (char *logname, char *outdir)
     if (!EnumResourceNamesA (NULL, "TESTRES", EnumTestFileProc, (LPARAM)&nr_of_files))
         report (R_FATAL, "Can't enumerate test files: %d",
                 GetLastError ());
-    wine_tests = heap_alloc (nr_of_files * sizeof wine_tests[0]);
+    wine_tests = xalloc(nr_of_files * sizeof wine_tests[0]);
 
     /* Do this only once during extraction (and version checking) */
     hmscoree = LoadLibraryA("mscoree.dll");
@@ -1181,8 +1235,8 @@ run_tests (char *logname, char *outdir)
     logfile = 0;
     if (newdir)
         remove_dir (tempdir);
-    heap_free(wine_tests);
-    heap_free(curpath);
+    free(wine_tests);
+    free(curpath);
 
     return logname;
 }
@@ -1228,7 +1282,7 @@ static void extract_only (const char *target_dir)
     if (!EnumResourceNamesA(NULL, "TESTRES", EnumTestFileProc, (LPARAM)&nr_of_files))
         report (R_FATAL, "Can't enumerate test files: %d", GetLastError ());
 
-    wine_tests = heap_alloc (nr_of_files * sizeof wine_tests[0] );
+    wine_tests = xalloc(nr_of_files * sizeof wine_tests[0] );
 
     report (R_STATUS, "Extracting tests");
     report (R_PROGRESS, 0, nr_of_files);
@@ -1272,6 +1326,7 @@ int __cdecl main( int argc, char *argv[] )
     int reset_env = 1;
     int poweroff = 0;
     int interactive = 1;
+    int prev_crash_dialog = 0;
     int i;
 
     InitCommonControls();
@@ -1292,12 +1347,7 @@ int __cdecl main( int argc, char *argv[] )
             exit (0);
         }
         else if ((argv[i][0] != '-' && argv[i][0] != '/') || argv[i][2]) {
-            if (nb_filters == ARRAY_SIZE(filters))
-            {
-                report (R_ERROR, "Too many test filters specified");
-                exit (2);
-            }
-            filters[nb_filters++] = argv[i];
+            add_filter( argv[i] );
         }
         else switch (argv[i][1]) {
         case 'c':
@@ -1334,6 +1384,7 @@ int __cdecl main( int argc, char *argv[] )
         case 'q':
             report (R_QUIET);
             interactive = 0;
+            quiet_mode++;
             break;
         case 's':
             if (!(submit = argv[++i]))
@@ -1421,6 +1472,8 @@ int __cdecl main( int argc, char *argv[] )
 
             if (!check_display_driver())
                 report (R_FATAL, "Unable to create a window, the display driver is not working.");
+
+            if (!interactive) prev_crash_dialog = disable_crash_dialog();
         }
 
         SetConsoleCtrlHandler(ctrl_handler, TRUE);
@@ -1436,7 +1489,7 @@ int __cdecl main( int argc, char *argv[] )
         if (nb_filters && !exclude_tests)
         {
             run_tests( logname, outdir );
-            exit(0);
+            exit( failures ? 3 : 0 );
         }
 
         while (!tag) {
@@ -1472,10 +1525,19 @@ int __cdecl main( int argc, char *argv[] )
             if (build_id[0] && nr_of_skips <= SKIP_LIMIT && failures <= FAILURES_LIMIT &&
                 !nr_native_dlls && !is_win9x &&
                 report (R_ASK, MB_YESNO, "Do you want to submit the test results?") == IDYES)
+            {
                 if (!send_file (submiturl, logname) && !DeleteFileA(logname))
                     report (R_WARNING, "Can't remove logfile: %u", GetLastError());
-        } else run_tests (logname, outdir);
-        report (R_STATUS, "Finished - %u failures", failures);
+                else
+                    failures = 0;  /* return success */
+            }
+        }
+        else
+        {
+            run_tests (logname, outdir);
+            report (R_STATUS, "Finished - %u failures", failures);
+        }
+        if (prev_crash_dialog) restore_crash_dialog( prev_crash_dialog );
     }
     if (poweroff)
     {
@@ -1493,5 +1555,5 @@ int __cdecl main( int argc, char *argv[] )
         }
         ExitWindowsEx(EWX_SHUTDOWN | EWX_POWEROFF | EWX_FORCEIFHUNG, SHTDN_REASON_MAJOR_OTHER | SHTDN_REASON_MINOR_OTHER);
     }
-    exit (0);
+    exit( failures ? 3 : 0 );
 }
